"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Setting_1 = require("@joplin/lib/models/Setting");
const test_utils_1 = require("@joplin/lib/testing/test-utils");
const Note_1 = require("@joplin/lib/models/Note");
const Folder_1 = require("@joplin/lib/models/Folder");
const ItemChange_1 = require("@joplin/lib/models/ItemChange");
const testUtils_1 = require("../../../testUtils");
const eventManager_1 = require("@joplin/lib/eventManager");
describe('JoplinWorkspace', () => {
    beforeEach(async () => {
        await (0, test_utils_1.setupDatabaseAndSynchronizer)(1);
        await (0, test_utils_1.switchClient)(1);
    });
    afterEach(async () => {
        await (0, test_utils_1.afterEachCleanUp)();
    });
    test('should listen to noteChange events', async () => {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        const appState = {
            selectedNoteIds: [],
        };
        const service = (0, testUtils_1.newPluginService)('1.4', {
            getState: () => {
                return appState;
            },
        });
        const pluginScript = (0, testUtils_1.newPluginScript)(`			
			joplin.plugins.register({
				onStart: async function() {
					await joplin.workspace.onNoteChange(async (event) => {
						await joplin.data.post(['folders'], null, { title: JSON.stringify(event) });
					});
				},
			});
		`);
        const note = await Note_1.default.save({});
        appState.selectedNoteIds.push(note.id);
        await ItemChange_1.default.waitForAllSaved();
        const plugin = await service.loadPluginFromJsBundle('', pluginScript);
        await service.runPlugin(plugin);
        await Note_1.default.save({ id: note.id, body: 'testing' });
        await ItemChange_1.default.waitForAllSaved();
        const folder = (await Folder_1.default.all())[0];
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        const result = JSON.parse(folder.title);
        expect(result.id).toBe(note.id);
        expect(result.event).toBe(ItemChange_1.default.TYPE_UPDATE);
        await service.destroy();
    });
    test('should return the selected folder', async () => {
        const service = (0, testUtils_1.newPluginService)();
        const pluginScript = (0, testUtils_1.newPluginScript)(`			
			joplin.plugins.register({
				onStart: async function() {
					const folder = await joplin.workspace.selectedFolder();
					await joplin.data.put(['folders', folder.id], null, { title: "changedtitle" });
				},
			});
		`);
        const folder = await Folder_1.default.save({ title: 'folder' });
        Setting_1.default.setValue('activeFolderId', folder.id);
        const plugin = await service.loadPluginFromJsBundle('', pluginScript);
        await service.runPlugin(plugin);
        const modFolder = await Folder_1.default.load(folder.id);
        expect(modFolder.title).toBe('changedtitle');
    });
    test('should remove event listeners when plugins are unloaded', async () => {
        const service = (0, testUtils_1.newPluginService)();
        const pluginScript = (0, testUtils_1.newPluginScript)(`
			joplin.plugins.register({
				onStart: async () => {
					// Register each listener 8 times to improve test reliability -- it's possible
					// for listeners for the same events to be added/removed by other sources.
					for (let i = 0; i < 8; i++) {
						await joplin.workspace.onNoteChange(async (event) => { });
						await joplin.workspace.onResourceChange(async (event) => { });
						await joplin.workspace.filterEditorContextMenu(async (event) => { });
					}
				}
			})
		`);
        const plugin = await service.loadPluginFromJsBundle('', pluginScript);
        await service.runPlugin(plugin);
        const itemChangeListenerCounter = eventManager_1.default.listenerCounter_(eventManager_1.EventName.ItemChange);
        const resourceChangeListenerCounter = eventManager_1.default.listenerCounter_(eventManager_1.EventName.ResourceChange);
        plugin.onUnload();
        expect(itemChangeListenerCounter.getCountRemoved()).toBeGreaterThanOrEqual(8);
        expect(resourceChangeListenerCounter.getCountRemoved()).toBeGreaterThanOrEqual(8);
        await service.destroy();
    });
});
//# sourceMappingURL=JoplinWorkspace.js.map