"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const locale_1 = require("../locale");
const ReportService_1 = require("./ReportService");
const test_utils_1 = require("../testing/test-utils");
const Folder_1 = require("../models/Folder");
const BaseItem_1 = require("../models/BaseItem");
const DecryptionWorker_1 = require("./DecryptionWorker");
const Note_1 = require("../models/Note");
const shim_1 = require("../shim");
const firstSectionWithTitle = (report, title) => {
    const sections = report.filter(section => section.title === title);
    if (sections.length === 0)
        return null;
    return sections[0];
};
const getCannotSyncSection = (report) => {
    return firstSectionWithTitle(report, (0, locale_1._)('Items that cannot be synchronised'));
};
const getIgnoredSection = (report) => {
    return firstSectionWithTitle(report, (0, locale_1._)('Ignored items that cannot be synchronised'));
};
const sectionBodyToText = (section) => {
    return section.body.map(item => {
        if (typeof item === 'string') {
            return item;
        }
        return item.text;
    }).join('\n');
};
describe('ReportService', () => {
    beforeEach(async () => {
        await (0, test_utils_1.setupDatabaseAndSynchronizer)(1);
        await (0, test_utils_1.setupDatabaseAndSynchronizer)(2);
        await (0, test_utils_1.switchClient)(1);
        // For compatibility with code that calls DecryptionWorker.instance()
        DecryptionWorker_1.default.instance_ = (0, test_utils_1.decryptionWorker)();
    });
    it('should move sync errors to the "ignored" section after clicking "ignore"', async () => {
        var _a, _b;
        const folder = await Folder_1.default.save({ title: 'Test' });
        const noteCount = 5;
        const testNotes = await (0, test_utils_1.createNTestNotes)(noteCount, folder);
        await (0, test_utils_1.synchronizerStart)();
        const disabledReason = 'Test reason';
        for (const testNote of testNotes) {
            await BaseItem_1.default.saveSyncDisabled((0, test_utils_1.syncTargetId)(), testNote, disabledReason);
        }
        const service = new ReportService_1.default();
        let report = await service.status((0, test_utils_1.syncTargetId)());
        // Items should all initially be listed as "cannot be synchronized", but should be ignorable.
        const unsyncableSection = getCannotSyncSection(report);
        const ignorableItems = [];
        for (const item of unsyncableSection.body) {
            if (typeof item === 'object' && item.canIgnore) {
                ignorableItems.push(item);
            }
        }
        expect(ignorableItems).toHaveLength(noteCount);
        expect(sectionBodyToText(unsyncableSection)).toContain(disabledReason);
        // Ignore all
        expect(await BaseItem_1.default.syncDisabledItemsCount((0, test_utils_1.syncTargetId)())).toBe(noteCount);
        expect(await BaseItem_1.default.syncDisabledItemsCountIncludingIgnored((0, test_utils_1.syncTargetId)())).toBe(noteCount);
        for (const item of ignorableItems) {
            await item.ignoreHandler();
        }
        expect(await BaseItem_1.default.syncDisabledItemsCount((0, test_utils_1.syncTargetId)())).toBe(0);
        expect(await BaseItem_1.default.syncDisabledItemsCountIncludingIgnored((0, test_utils_1.syncTargetId)())).toBe(noteCount);
        await (0, test_utils_1.synchronizerStart)();
        report = await service.status((0, test_utils_1.syncTargetId)());
        // Should now be in the ignored section
        const ignoredSection = getIgnoredSection(report);
        expect(ignoredSection).toBeTruthy();
        expect(sectionBodyToText(unsyncableSection)).toContain(disabledReason);
        expect(sectionBodyToText(getCannotSyncSection(report))).not.toContain(disabledReason);
        // Should not be possible to re-ignore an item in the ignored section
        let ignoredItemCount = 0;
        for (const item of ignoredSection.body) {
            if (typeof item === 'object' && ((_a = item.text) === null || _a === void 0 ? void 0 : _a.includes(disabledReason))) {
                expect(item.canIgnore).toBeFalsy();
                expect(item.canRetry).toBe(true);
                ignoredItemCount++;
            }
        }
        // Should have the correct number of ignored items
        expect(await BaseItem_1.default.syncDisabledItemsCountIncludingIgnored((0, test_utils_1.syncTargetId)())).toBe(ignoredItemCount);
        expect(ignoredItemCount).toBe(noteCount);
        // Clicking "retry" should un-ignore
        for (const item of ignoredSection.body) {
            if (typeof item === 'object' && ((_b = item.text) === null || _b === void 0 ? void 0 : _b.includes(disabledReason))) {
                expect(item.canRetry).toBe(true);
                await item.retryHandler();
                break;
            }
        }
        expect(await BaseItem_1.default.syncDisabledItemsCountIncludingIgnored((0, test_utils_1.syncTargetId)())).toBe(noteCount - 1);
    });
    it('should support ignoring sync errors for resources that failed to download', async () => {
        const createAttachmentDownloadError = async () => {
            await (0, test_utils_1.switchClient)(2);
            const note1 = await Note_1.default.save({ title: 'note' });
            await shim_1.default.attachFileToNote(note1, `${test_utils_1.supportDir}/photo.jpg`);
            await (0, test_utils_1.synchronizerStart)();
            await (0, test_utils_1.switchClient)(1);
            const previousMax = (0, test_utils_1.synchronizer)().maxResourceSize_;
            (0, test_utils_1.synchronizer)().maxResourceSize_ = 1;
            await (0, test_utils_1.synchronizerStart)();
            (0, test_utils_1.synchronizer)().maxResourceSize_ = previousMax;
        };
        await createAttachmentDownloadError();
        const service = new ReportService_1.default();
        let report = await service.status((0, test_utils_1.syncTargetId)());
        const unsyncableSection = getCannotSyncSection(report);
        expect(sectionBodyToText(unsyncableSection)).toContain('could not be downloaded');
        // Item for the download error should be ignorable
        const ignorableItems = [];
        for (const item of unsyncableSection.body) {
            if (typeof item === 'object' && item.canIgnore) {
                ignorableItems.push(item);
            }
        }
        expect(ignorableItems).toHaveLength(1);
        await ignorableItems[0].ignoreHandler();
        // Should now be ignored.
        report = await service.status((0, test_utils_1.syncTargetId)());
        const ignoredItem = getIgnoredSection(report).body.find(item => typeof item === 'object' && item.canRetry === true);
        expect(ignoredItem).not.toBeFalsy();
        // Type narrowing
        if (typeof ignoredItem === 'string')
            throw new Error('should be an object');
        // Should be possible to retry
        await ignoredItem.retryHandler();
        await (0, test_utils_1.synchronizerStart)();
        // Should be fixed after retrying
        report = await service.status((0, test_utils_1.syncTargetId)());
        expect(getIgnoredSection(report)).toBeNull();
        expect(getCannotSyncSection(report)).toBeNull();
    });
});
//# sourceMappingURL=ReportService.test.js.map