"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseEnv = void 0;
const fs_extra_1 = require("fs-extra");
const path_1 = require("path");
const parseEnv = (env_file, options = {}) => {
    options = Object.assign({ logger: console, overwrite: false, raise: true, verbose: false }, options);
    if (typeof env_file !== 'string') {
        if (options.raise) {
            throw new TypeError("Environment file argument is not a valid `String`: " + env_file);
        }
        else {
            if (options.verbose && options.logger) {
                options.logger.error('[ENV]: ERROR Environment file argument is not a valid `String`:', process.env.ENV_FILE);
            }
            return {};
        }
    }
    try {
        env_file = process.env.ENV_FILE = (0, path_1.resolve)(env_file) || process.env.ENV_FILE;
    }
    catch (err) {
        if (options.raise) {
            throw new TypeError("Environment file path could not be resolved: " + err);
        }
        else {
            if (options.verbose && options.logger) {
                options.logger.error('[ENV]: ERROR Environment file path could not be resolved:', process.env.ENV_FILE);
            }
            return {};
        }
    }
    const data = {};
    data[env_file] = {};
    if (options.verbose && options.logger) {
        options.logger.info('[ENV]: Loading environment:', env_file);
    }
    if ((0, fs_extra_1.existsSync)(env_file)) {
        let lines = [];
        try {
            lines = ((0, fs_extra_1.readFileSync)(env_file, 'utf8') || '')
                .split(/\r?\n|\r/)
                .filter(function (line) {
                return /\s*=\s*/i.test(line);
            })
                .map(function (line) {
                return line.replace('exports ', '');
            });
        }
        catch (err) {
            if (options.raise) {
                throw new TypeError("Environment file could not be read: " + err);
            }
            else {
                if (options.verbose && options.logger) {
                    options.logger.error('[ENV]: ERROR Environment file could not be read:', env_file);
                }
                return {};
            }
        }
        const lineComments = [];
        const lineVariables = [];
        let is_comment = false;
        lines.forEach(function (line) {
            is_comment = /^\s*\#/i.test(line); // ignore comment lines (starting with #).
            if (is_comment) {
                lineComments.push(line);
                if (options.verbose && options.logger) {
                    options.logger.info('[ENV]: Ignored line:', line);
                }
            }
            else {
                lineVariables.push(line);
                const key_value = line.match(/^([^=]+)\s*=\s*(.*)$/);
                const env_key = key_value[1];
                // remove ' and " characters if right side of = is quoted
                const env_value = key_value[2].match(/^(['"]?)([^\n]*)\1$/m)[2];
                // overwrite already defined `process.env.*` values?
                if (!!options.overwrite) {
                    data[env_file][env_key] = env_value;
                    if (options.verbose && options.logger && data[env_file][env_key] !== env_value) {
                        options.logger.info('[ENV]: Overwritten ', data[env_file][env_key], ' => ', env_value);
                    }
                }
                else {
                    data[env_file][env_key] = process.env[env_key] || env_value;
                }
                process.env[env_key] = data[env_file][env_key];
                if (options.verbose && options.logger) {
                    options.logger.info('[ENV]:', data[env_file]);
                }
            }
        });
    }
    else {
        if (options.raise) {
            throw new TypeError("Environment file doesn't exist: " + env_file);
        }
        else {
            if (options.verbose && options.logger) {
                options.logger.error('[ENV]: ERROR Environment file path could not be resolved:', env_file);
            }
            return {};
        }
    }
    return data[env_file];
};
exports.parseEnv = parseEnv;
//# sourceMappingURL=parseEnv.js.map